﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Collections.Specialized;
using System.Configuration.Provider;
using mobx.mobi.DAL;
using mobx.mobi.DAL.Managers;

namespace mobx.mobi
{
    public class MbRoleProvider : RoleProvider
    {
        #region Fields
        private string _appName;
        #endregion

        #region Methods
        /// <summary>
        /// Adds the specified user names to the specified roles for the configured applicationName
        /// </summary>
        /// <param name="usernames">A string array of user names to be added to the specified roles.</param>
        /// <param name="roleNames">A string array of the role names to add the specified user names to.</param>
        public override void AddUsersToRoles(string[] usernames, string[] roleNames)
        {
            if (usernames != null &&
                usernames.Length > 0 &&
                roleNames != null &&
                roleNames.Length > 0)
            {
                UserManager userManager = new UserManager();
                RoleManager roleManager = new RoleManager();
                foreach (string userName in usernames)
                {
                    //find user by username
                    MbUser user = userManager.GetUserByEmail(userName);
                    if (user != null)
                    {
                        List<int> roleIds = new List<int>();
                        //find roleIds by rolename
                        foreach (string roleName in roleNames)
                        {
                            MbUserRole role = roleManager.GetRoleByName(roleName);
                            if (role != null)
                            {
                                roleIds.Add(role.UserRoleId);
                            }
                        }
                        if (roleIds.Count > 0)
                        {
                            roleManager.AddUserToRoles(user.UserId, roleIds);
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Adds a new role to the data source for the configured applicationName.
        /// </summary>
        /// <param name="roleName">The name of the role to create.</param>
        public override void CreateRole(string roleName)
        {
            RoleManager roleManager = new RoleManager();
            roleManager.AddRole(roleName);
        }

        /// <summary>
        /// Removes a role from the data source for the configured applicationName.
        /// </summary>
        /// <param name="roleName">The name of the role to delete.</param>
        /// <param name="throwOnPopulatedRole">If true, throw an exception if roleName has one or more members and do not delete roleName.</param>
        /// <returns>true if the role was successfully deleted; otherwise, false.</returns>
        public override bool DeleteRole(string roleName, bool throwOnPopulatedRole)
        {
            bool ret = false;
            bool canDelete = false;
            if (throwOnPopulatedRole)
            {
                string[] usersInRole = GetUsersInRole(roleName);
                if (usersInRole != null &&
                    usersInRole.Length > 0)
                {
                    throw new Exception("Cannot delete the role. There are users in this role");
                    ret = false;
                }
                else
                {
                    canDelete = true;
                }
            }
            else
            {
                canDelete = true;
            }

            if (canDelete)
            {
                RoleManager roleManager = new RoleManager();
                ret = roleManager.DeleteRole(roleName);
            }
            return ret;
        }

        /// <summary>
        /// Gets an array of user names in a role where the user name contains the specified user name to match.
        /// </summary>
        /// <param name="roleName">The role to search in.</param>
        /// <param name="usernameToMatch">The user name to search for.</param>
        /// <returns>A string array containing the names of all the users where the user name matches usernameToMatch and the user is a member of the specified role.</returns>
        public override string[] FindUsersInRole(string roleName, string usernameToMatch)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Gets a list of all the roles for the configured applicationName.
        /// </summary>
        /// <returns>A string array containing the names of all the roles stored in the data source for the configured applicationName.</returns>
        public override string[] GetAllRoles()
        {
            RoleManager roleManager = new RoleManager();
            var allRoles = roleManager.GetAllRoles();
            if (allRoles != null &&
                allRoles.Any())
            {
                return allRoles.Select(x=>x.Name).ToArray();
            }
            return null;
        }

        /// <summary>
        /// Gets a list of the roles that a specified user is in for the configured applicationName.
        /// </summary>
        /// <param name="username">The user to return a list of roles for.</param>
        /// <returns>A string array containing the names of all the roles that the specified user is in for the configured applicationName.</returns>
        public override string[] GetRolesForUser(string username)
        {
            var roles = new List<string>();
            UserManager userManager = new UserManager();
            MbUser user = userManager.GetUserByEmail(username);
            if (user == null)
            {
                return roles.ToArray();
            }
            else
            {
                RoleManager roleManager = new RoleManager();
                var customerRoles = roleManager.GetUserRolesByUserId(user.UserId, false);
                foreach (var cr in customerRoles)
                {
                    if (cr.Active)
                    {
                        if (!roles.Contains(cr.Name))
                            roles.Add(cr.Name);
                    }
                }
            }
            return roles.ToArray();
        }

        /// <summary>
        /// Gets a list of users in the specified role for the configured applicationName.
        /// </summary>
        /// <param name="roleName">The name of the role to get the list of users for.</param>
        /// <returns>A string array containing the names of all the users who are members of the specified role for the configured applicationName.</returns>
        public override string[] GetUsersInRole(string roleName)
        {
            RoleManager roleManager = new RoleManager();
            List<MbUser> userList = roleManager.GetUsersInRole(roleName, true);
            if (userList != null &&
                userList.Any())
            {
                return userList.Select(u=>u.Email).ToArray();
            }
            return null;
        }

        /// <summary>
        /// Initializes the provider.
        /// </summary>
        /// <param name="name">The friendly name of the provider.</param>
        /// <param name="config">A collection of the name/value pairs representing the provider-specific attributes specified in the configuration for this provider.</param>
        public override void Initialize(string name, NameValueCollection config)
        {
            if (config == null)
            {
                throw new ArgumentNullException("config");
            }
            if (string.IsNullOrEmpty(name))
            {
                name = "CmsRoleProvider";
            }
            if (string.IsNullOrEmpty(config["description"]))
            {
                config.Remove("description");
                config.Add("description", "Roles Provider");
            }
            base.Initialize(name, config);
            this._appName = config["applicationName"];
            if (string.IsNullOrEmpty(this._appName))
            {
                this._appName = "mobx.mobi";
            }
            if (this._appName.Length > 0x100)
            {
                throw new ProviderException("Provider application name too long");
            }
            config.Remove("applicationName");

            this._appName = "mobx.mobi";

            config.Remove("connectionStringName");

            if (config.Count > 0)
            {
                string text2 = config.GetKey(0);
                if (!string.IsNullOrEmpty(text2))
                {
                    throw new ProviderException(string.Format("Provider unrecognized attribute {0}", text2));
                }
            }
        }

        /// <summary>
        /// Gets a value indicating whether the specified user is in the specified role for the configured applicationName.
        /// </summary>
        /// <param name="username">The user name to search for.</param>
        /// <param name="roleName">The role to search in.</param>
        /// <returns>true if the specified user is in the specified role for the configured applicationName; otherwise, false.</returns>
        public override bool IsUserInRole(string username, string roleName)
        {
            if (string.IsNullOrEmpty(roleName))
                return false;

            UserManager userManager = new UserManager();
            MbUser user = userManager.GetUserByEmail(username);

            if (user == null)
            {
                return false;
            }
            RoleManager roleManager = new RoleManager();
            var customerRoles = roleManager.GetUserRolesByUserId(user.UserId, false);
            foreach (var cr in customerRoles)
            {
                if (cr.Active)
                {
                    if (roleName.ToLowerInvariant() == cr.Name.ToLowerInvariant())
                        return true;
                }
            }

            return false;
        }

        /// <summary>
        /// Removes the specified user names from the specified roles for the configured applicationName.
        /// </summary>
        /// <param name="usernames">A string array of user names to be removed from the specified roles.</param>
        /// <param name="roleNames">A string array of role names to remove the specified user names from.</param>
        public override void RemoveUsersFromRoles(string[] usernames, string[] roleNames)
        {
            if (usernames != null &&
               usernames.Length > 0 &&
               roleNames != null &&
               roleNames.Length > 0)
            {
                UserManager userManager = new UserManager();
                RoleManager roleManager = new RoleManager();
                foreach (string userName in usernames)
                {
                    //find user by username
                    MbUser user = userManager.GetUserByEmail(userName);
                    if (user != null)
                    {
                        List<int> roleIds = new List<int>();
                        //find roleIds by rolename
                        foreach (string roleName in roleNames)
                        {
                            MbUserRole role = roleManager.GetRoleByName(roleName);
                            if (role != null)
                            {
                                roleIds.Add(role.UserRoleId);
                            }
                        }
                        if (roleIds.Count > 0)
                        {
                            roleManager.RemoveRolesFromUser(user.UserId, roleIds);
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Gets a value indicating whether the specified role name already exists in the role data source for the configured applicationName.
        /// </summary>
        /// <param name="roleName">The name of the role to search for in the data source.</param>
        /// <returns>true if the role name already exists in the data source for the configured applicationName; otherwise, false.</returns>
        public override bool RoleExists(string roleName)
        {
            return GetAllRoles().Where(x => x==roleName).Any();
        }
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets the name of the application to store and retrieve role information for.
        /// </summary>
        public override string ApplicationName
        {
            get
            {
                return this._appName;
            }
            set
            {
                this._appName = value;
            }
        }
        #endregion
    }
}
